clear all
macro drop all
capture log close 
set more off
set matsize 10000
set linesize 120
set pagesize 80
set seed 1111
set maxvar 10000
program drop _all
clear mata
version 16
 
**cd "C:\Users\Administrator\Dropbox\Linearize_BB\JORS_Published\stata_final"
cd "C:\Users\mpfaf\Dropbox\Linearize_BB\JORS_Published\stata_final"

/*******************************************************************************/
/* Globals */
/*******************************************************************************/
global b=42 /* 42 wiod countries */
global sig=6.982
global groups=3
global group_out=11

global K=5
global s=5
global eqs=(2*$b-1)
global dummies="ibn.im ib$b.ex"
global dm="ibn.im"
global dx="ib$b.ex"
global re=  "border ldist  contig comlang rta "

/*******************************************************************************/
/* Experiment */
/*******************************************************************************/ 
/*ldist border rta*/
**local exp="rta"
local exp="rta"

** qui{
/*******************************************************************************/ 
/* Data file */
/*******************************************************************************/
use Linearize_bb_wiod, clear

/*******************************************************************************/
/* Log file */
/*******************************************************************************/
log using Linearize_bb_wiod_decomposition_`exp',replace

/*******************************************************************************/
/* Final set of variables */
/*******************************************************************************/
keep  ex im exname imname  yi yj border ldist contig  comlang colony rta cu fta  lim lex

/*******************************************************************************/
/* Initializing Variables */
/*******************************************************************************/
gen borderc=0
gen double one=1
gen double za0=0
gen double zac=0
gen double xbb=0
gen double xcbb=0
gen double yp_bb=0
gen double yp_bvols=0
gen double yp_bb_ww=0

gen double ypcf_avw=0
gen double ypcf_avwge=0
gen double ypcf_bb=0
gen double ypcf_bvols=0
gen double ypcf_ww=0
gen double ypcf_wwge=0
gen double psls=0

gen double yp_avw_w=0
gen double yp_bb_w=0
gen double yp_bvols_w=0
gen double ypcf_w=0
gen double yp_ww_w=0
gen double yp_w=0
gen double sp_bbt=0

gen double ypcf_bb_w=0
gen double ypcf_bvols_w=0
gen double ypcf_ww_w=0
gen double ypcf_avw_w=0
gen double ypcf_avwge_w=0
gen double ypcf_wwge_w=0

gen double dypcf_avw=0
gen double dypcf_avwge=0
gen double dypcf_bb=0
gen double dypcf_ww=0
gen double dypcf_wwge=0

gen double dypcf_avw_w=0
gen double dypcf_avwge_w=0
gen double dypcf_bb_w=0
gen double dypcf_ww_w=0
gen double dypcf_wwge_w=0

gen double dypcf_dir_w=0
gen double dypcf_dir_avw=0

gen double ypbb_avwge=0
gen double ypt_avw=0

gen double r_bvols=0
gen double r_bvols1=0
gen double r_bvols2=0
gen double r_bvols3=0
gen double r_bb=0
gen double r_bb1=0
gen double r_bb2=0

/*******************************************************************************/
/*Counterfactuals */
/*******************************************************************************/
di as red "experimient: "  "`exp'"
if "`exp'"=="ldist" {
gen ldistcf=ldist
replace ldistcf=ldist-log(0.90)/1.2111451274 if ex~=im
global recf=" border ldistcf  contig comlang rta "
}

if "`exp'"=="border" {
gen bordercf=0
global recf=" bordercf ldist  contig comlang rta "
}

if "`exp'"=="rta" {
gen rtacf=0
global recf=" border ldist  contig comlang rtacf "
}

/*******************************************************************************
/* Descriptives */
********************************************************************************/
sum  $re
sum  $re  if ex~=im
sum  $re  if ex==im

/*******************************************************************************/
/* 0. true model is PPML*/
/*******************************************************************************/
gen thx=yi
gen thm=yj
gen thxm=yi*yj

table ex , c(sum   thxm  m thx ) row
table im , c(sum   thxm  m thm ) row

/*
*** simulates parameters ***
1	-2.145215	-2.144369	-2.607016	-2.401684	-2.144871	-2.154290
2	-0.833723	-0.834228	-0.905588	-0.911525	-0.833247	-0.832330
3	-0.287841	-0.285738	-0.133215	-0.174538	-0.287082	-0.284418
4	-0.326669	-0.327888	-0.198584	-0.241149	-0.327542	-0.324533
5	 0.589344	0.591799	0.708167	0.704003	0.589925	0.591478
*/

# delimit ; 
mat B=( -2.145215,	-2.144369,	-2.607016,	-2.401684,	-2.144871,	-2.154290 \
		-0.833723,	-0.834228,	-0.905588,	-0.911525,	-0.833247,	-0.832330 \
		-0.287841,	-0.285738,	-0.133215,	-0.174538,	-0.287082,	-0.284418 \
		-0.326669,	-0.327888,	-0.198584,	-0.241149,	-0.327542,	-0.324533 \
		 0.589344, 	 0.591799,	 0.708167,	 0.704003,	 0.589925,	 0.591478 
);
# delimit cr 

mat li B

mat b_t  =B[1..5,1]
mat b_avw=B[1..5,2]
mat b_bb =B[1..5,3]   /*true  GDP share -> changed in mc file*/
mat b_bvols =B[1..5,4]  /*observed  GDP share -> changed in mc file*/
mat b_w  =B[1..5,5]
mat b_ww =B[1..5,6]

mat li b_t
mat li b_bb
mat li b_bvols


/*******************************************************************************/
/* Initializing linear predictors */
/*******************************************************************************/
gen zat=0
gen za_t=0
gen za_avw=0
gen za_bvols=0
gen za_bb=0
gen za_w=0
gen za_ww=0

gen zacf_t=0
gen zacf_avw=0
gen zacf_bvols=0
gen zacf_bb=0
gen zacf_w=0
gen zacf_ww=0


local i=1
qui foreach z of global re {
display "`z'"
replace za_t=za_t+B[`i',1]*`z'
replace za_avw=za_avw+B[`i',2]*`z'
replace za_bvols=za_bvols+B[`i',4]*`z'
replace za_bb= za_bb +B[`i',3]*`z'
replace za_w=za_w    +B[`i',5]*`z'
replace za_ww=za_ww  +B[`i',6]*`z'
local i=`i'+1
}


local i=1
foreach z of global recf {
display "`z'"
replace zacf_t=zacf_t+B[`i',1]*`z'
replace zacf_avw=zacf_avw+B[`i',2]*`z'
replace zacf_bvols=zacf_bvols+B[`i',4]*`z'
replace zacf_bb= zacf_bb +B[`i',3]*`z'
replace zacf_w=zacf_w    +B[`i',5]*`z'
replace zacf_ww=zacf_ww  +B[`i',6]*`z'
local i=`i'+1
}

/*******************************************************************************/
*** starting values true models ***
glm thxm ibn.im ib$b.ex , nocons family(poisson) robust offset(za_t)
/*******************************************************************************/predict sp
qui sum sp,d
replace sp=sp/r(sum)

drop thx thm thxm yi yj
egen thx=sum(sp),by(ex)
egen thm=sum(sp),by(im)
gen  thxm=thx*thm
gen yi=thx
gen yj=thm
gen ls=log(sp)-log(yi)-log(yj)
gen yt=ls
gen y=ls
gen s=exp(ls+log(yi)+log(yj))

table ex , c(sum   thxm  m thx sum sp sum s) row
table im , c(sum   thxm  m thm sum sp sum s) row
/*******************************************************************************/

/*******************************************************************************/
/* Mata invariant matrices and procedures */
/*******************************************************************************/
sort ex im 
do mata_mat_wiod.do
do mata_prod_wiod_final.do
/*******************************************************************************/

/*******************************************************************************
/* Check solver for conditional GE at true parameters */
********************************************************************************/
mata 
za=za_t

p=pass()
p.za=za
p.thphi=thphi
p.D=D
p.yi=yi
p.yj=yj
end

*** Starting values from Poisson when solving for mui and mj
qui poisson sp $re  ibn.im ib$b.ex , nocons
mat b_start=e(b)'
mata
b_start=st_matrix( "b_start")
start=b_start[$K+1..rows(b_start)-1,1]

S = solvenl_init()
solvenl_init_argument(S,1, p)
solvenl_init_evaluator(S, &avw())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.075)
solvenl_init_startingvals(S, start)
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-20)
solvenl_init_conv_nearzero(S, 1e-20)
solvenl_init_iter_log(S, "on")
v = solvenl_solve(S)
ypt_avw=za+D*v
norm(D'*exp(ypt_avw)-thphi)
st_store(. , "ypt_avw",(ypt_avw))
end
gen spt_avw=exp(ypt_avw)
egen yit=sum(spt_avw),by(ex)
egen yjt=sum(spt_avw),by(im)

table ex if ex==im, c(sum yit sum yi) row
table im if ex==im, c(sum yjt sum yj) row

/*******************************************************************************/
mata mu=v[43..83,1]
mata m=v[1..42,1]
mata thphix= thphi[43..83,1]
mata (mu, thphix)
mata sum(thphix)
mata thphix_base=1-sum(thphix)
mata thphix
mata mu=(mu\0)
mata thphix=(thphix\thphix_base)
mata (mu, thphix)

mata sigma = 5
mata p_avw=exp(mu/(1-sigma))
mata a=thphix:/p_avw
mata a=a
mata (p_avw,a,p_avw:*a-thphix, thxx)
/*******************************************************************************/

/*******************************************************************************/

/*******************************************************************************/
/* BB full ge at approximazion point t=1*/
/*******************************************************************************/
mata 
zabb=J(rows(Z),1,0)
start=log(thm\thx)
mu0=start[$b+1..rows(start),1]
mu0=(mu0\0)
end

/*******************************************************************************/
/* counterfactual at t=1 full ge*/
/*******************************************************************************/
mata 
pge=pass_ge()
pge.za=zabb
pge.thphi=thphi
pge.D=D
pge.Dm=Dm
pge.Dxx=Dxx
pge.yi=yi
pge.yj=yj
pge.thxx=thxx  /*plug in endowmewnts */
pge.mu0=mu0
pge.s=$s
pge.thm=thm

/*solve ge counterfactual*/
S = solvenl_init()
solvenl_init_argument(S,1, pge)
solvenl_init_evaluator(S, &avwge())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.05)
solvenl_init_startingvals(S, start )
solvenl_init_conv_maxiter(S, 1000)
solvenl_init_conv_iterchng(S,  1e-25)
solvenl_init_conv_nearzero(S, 1e-25)
vbbge = solvenl_solve(S)
ypbb_avwge=zabb+D*vbbge
sp_bbt=exp(D*vbbge)
st_store(. , "ypbb_avwge",(ypbb_avwge)) 
st_store(. , "sp_bbt",(sp_bbt)) 
end

gen double spbb_avwge=exp(ypbb_avwge)
egen double yibb=sum(spbb_avwge),by(ex)
egen double yjbb=sum(spbb_avwge),by(im)

table ex if ex==im, c(sum yibb sum yi) row
table im if ex==im, c(sum yjbb sum yj) row

/*******************************************************************************/
mata mubb=vbbge[43..83,1]
mata mbb=vbbge[1..42,1]

mata thphixbb= Dxx'*sp_bbt /*thphixbb determined endogenously*/
mata mubb=(mubb\0)

mata yitr=Dx'yi/42
mata yitr_base=1-sum(yitr)
mata yitr=(yitr\yitr_base)

mata p_bvols=exp(mubb/(1-sigma))

mata e=J(42,1,1)
mata  (thphix, Dxx'(exp(mu)#exp(m)), exp(mu)*1, Dxx'(e#exp(m)) ,  exp(mu):*Dxx'(e#exp(m)) -  Dxx'(exp(mu)#exp(m)))

mata  (thphixbb, exp(mubb)*.2105071404, Dxx'(e#exp(mbb)) ,  exp(mubb):*Dxx'(e#exp(mbb)) -  Dxx'(exp(mubb)#exp(mbb)))

mata ((Dxx'*yi/42):/ thphixbb)
/*******************************************************************************/

mata sigma = 5
mata p_bvols=exp(mubb/(1-sigma))
mata a=thphixbb:/p_bvols
mata a=a
mata (p_bvols,a,p_bvols:*a-thphixbb) /* bvols is correct */

mata astar=(p_bvols:/p_avw):*a
mata astar1=( exp(mubb/(1-sigma)):/exp(mu/(1-sigma)) ):*a
mata (astar,a, astar1:/a)
mata (p_bvols:*a,  thphixbb, p_avw:*astar,thphixbb)
mata ((p_bvols:/p_avw),p_bvols:*a,  thphixbb, p_avw:*astar1,thphixbb, astar-a)
preserve
clear
set obs 42
gen a=0
gen astar=0
mata st_store(. , "a",(a)) 
mata st_store(. , "astar",(astar)) 

mata (sum(a), sum(astar), min(astar-a), mean(astar-a), max(astar-a))
gen da=astar-a
sum

**hist da, bin(20)

label var a "A" 
label var  da"A*-A"

/*
scatter da a, ///
		ytitle("A*-A",size(small) margin(medium)) ///
		xtitle("True endowment, A",size(small) margin(medium)) ///
		msize(small small small small ) ///
        msymbol(o o o o) mlwidth( vthin vthin vthin vthin  ) ///
        mfcolor(gs8 gs16) mlcolor(gs1 gs5 gs5 gs5)   yline(0,  lcolor(gs1) )  ///		  
		legend( rows(1)  region(lstyle(none)) size(small) ) scheme( s1mono)  ///
		xsize(4) ysize(4)


*/
restore
/******************************************************************************/

/*******************************************************************************/
/* 1. AVW in levels*/
/*******************************************************************************/
*** check AV-estimates
qui glm thxm ibn.im ib$b.ex  , nocons offset(za_avw) family(poisson) irls vce(robust) 
predict thxmp
gen yp_avw=log(thxmp)

table ex , c(sum thxmp sum thxm m thx) row
table im , c(sum thxmp sum thxm m thm) row

** b_avw from Monet Carlos
mat outb=  (b_w,  b_avw )
mat li outb
mata yp_avw=st_data(., "yp_avw")
/*******************************************************************************/

/*******************************************************************************/
/* 2. Solve AVW using b_w  */
/*******************************************************************************/
*** za = Index from structural parameters za_w
*** Solve AVW-trade resistance equations at yi and yj					
glm thxm ibn.im ib$b.ex  , nocons offset(za_w) family(poisson) irls vce(robust) 
predict sp_w
gen y_awv_w=log(sp_w)
mat mu1=e(b)'
mata mu1=st_matrix( "mu1")

table ex , c(sum sp_w sum thxmp sum thxm m thx) row
table im , c(sum sp_w sum thxmp sum thxm m thm) row

mata yp_avw_w=st_data(., "yp_avw_w")
mata sp_w=st_data(., "sp_w")
/*******************************************************************************/

/*******************************************************************************
/* 3. Base and counterfactuals with linear approximation */
********************************************************************************/

/*******************************************************************************/
/* intermediate step: Dummy model */
/*******************************************************************************/
qui reg ls $re  ibn.im ib$b.ex , nocons r
predict pls_w
predict rr, res
gen  y_fe=pls_w+log(yi)+log(yj)

*** within-parameters long 
mat b_w1=e(b)'
mata b_w1=st_matrix( "b_w1")
mata b_w1=b_w1[1.. rows(b_w1)-1,1]


/*yi yj avw salesand expenditure shares at true t */
/*sp_bbt avw sales and expenditure shares at t=0 */
mata 
b_avw=st_matrix("b_avw")
b_wz=b_w[1..$K,1]
b_we=st_matrix("b_we")
b_bb=st_matrix("b_bb")
b_bvols=st_matrix("b_bvols")
b_ww=st_matrix( "b_ww")

Gbb=diag(sp_bbt)
Gbvols=diag(yi:*yj)
Gw=diag(sp_w)
Qbb=I($b^2)- D*invsym(D'*Gbb*D)*D'*Gbb
Qbvols=I($b^2)- D*invsym(D'*Gbvols*D)*D'*Gbvols 
Qw=I($b^2)- D*invsym(D'*D)*D' 
Qww=I($b^2)- D*invsym(D'*Gw*D)*D'*Gw

mu=b_w1[$b+$K+1..rows(b_w1),1]
mu=(mu\0)
	
/*
/*******************************************************************************/
/* call mata procedure for error decomposition  */
/*******************************************************************************/
			   
dypcf= predict(Qbb, Qbvols, Qw, W, Wcf,  Z,  Zcf, Z0, Z0cf, sp, sp_w, sp_bbt, D,  ///
               b_w, /// 
			   za_avw,  za_bb, za_bvols, za_w, za_ww, ///
			   zacf_avw,  zacf_bb, zacf_bvols, zacf_w, zacf_ww, ///			   
			   thm, thx,thxx, y, yi, yj, mu )

*/
/*******************************************************************************/
/* call mata procedure for decomposition */
/*******************************************************************************/
/*
unction decom  (real matrix Qbb, matrix Qbvols, real matrix Qw, 
	real matrix Z, real matrix Z0,
	real matrix ypt_avw, real matrix sp, real matrix sp_w, real matrix sp_bbt, 
	real matrix D, real matrix b_w,
	real matrix za_t, real matrix za_avw, real matrix za_bb,  real matrix za_bba, 
	real matrix za_w, real matrix za_ww, 
	real matrix zacf_avw, real matrix zacf_bb,  real matrix zacf_bba, 
	real matrix zacf_w, real matrix zacf_ww,
	real matrix thm , real matrix thx, real matrix thxx, 
	real matrix y, real matrix yi ,  real matrix yj,  real vector mu , 
	struct  decomp scalar v )
*/

v= decomp()
decom(Qbb, Qbvols, Qw,  Z,  Z0, ///
     ypt_avw,sp, sp_w, sp_bbt, D, b_w, /// 
	 za_t, za_avw,  za_bb, za_bvols, za_w, za_ww, ///
	 zacf_avw,  zacf_bb, zacf_bvols, zacf_w, zacf_ww, ///			   
	 thm, thx,thxx, y, yi, yj,mu,v )

r_bb=v.r_bb
r_bb1=v.r_bb1
r_bb2=v.r_bb2
r_bvols=v.r_bvols 
r_bvols1=v.r_bvols1 
r_bvols2=v.r_bvols2 
r_bvols3=v.r_bvols3
		   
st_store(. , "r_bvols", r_bvols)
st_store(. , "r_bvols1", r_bvols1)
st_store(. , "r_bvols2", r_bvols2)
st_store(. , "r_bvols3", r_bvols3)
st_store(. , "r_bb", r_bb)
st_store(. , "r_bb1", r_bb1)
st_store(. , "r_bb2", r_bb2)
end

sum s*
sum y*
sum r*

********************************************************************************
*** Table 2
********************************************************************************
xtile qypt_avw=ypt_avw, nquantiles(5)
xtile qza_t=za_t, nquantiles(5)

version 17
/* Label according to text Corollary */
label var ypt_avw  "xtilde"
label var qypt_avw  "quantiles of xtilde"

table qypt_avw, stat(mean ypt_avw r_bvols r_bb) nformat(%5.3f) 
table qypt_avw, stat(mean r_bb r_bb1 r_bb2 ) nformat(%5.3f)

*** Table 2 part 2 *100
table qypt_avw, stat(mean r_bvols1 r_bvols2 r_bvols3) nformat(%5.3f)

gen er_bvols=100*(exp(r_bvols)-1)
gen er_bb=100*(exp(r_bb)-1)

label var ypbb_avwge  "xtilde_BB"
label var er_bvols "r_bvols"
label var er_bb  "r_rbb"

*** Table 2 part 1
table qypt_avw, stat(mean ypt_avw er_bb er_bvols) nformat(%5.3f) 

exit
