clear all
macro drop all
capture log close 
set more off
set matsize 10000
set linesize 120
set pagesize 80
set seed 1111
set maxvar 10000
program drop _all
set matastrict off
clear mata
version 16
 
cd "C:\Users\Administrator\Dropbox\Linearize_BB\JORS_Published\stata_final"

/*******************************************************************************/
/* Globals */
/*******************************************************************************/
global b=42 /* 42 wiod countries */
global sig=6.982
global groups=3
global group_out=11

global K=5
global s=5
global eqs=(2*$b-1)
global dummies="ibn.im ib$b.ex"
global dm="ibn.im"
global dx="ib$b.ex"
global re=  "border ldist  contig comlang rta "
**global sc=0.000000001
global sc=0.5

/*******************************************************************************/
/* Experiment */
/*******************************************************************************/ 
/*ldist tc*/
local exp="tc"
**local exp="ldist"
di as red "`exp'"

if "`exp'" == "tc" {
/* 0.95 0.75 */
global cf=0.75
local cf=$cf
di as red "`exp'"
}

if "`exp'"=="ldist" {
/* 0.95 0.75 */
global cf=0.95
local cf=$cf

di as red "`exp'"
}

** qui{
/*******************************************************************************/ 
/* Data file */
/*******************************************************************************/
use Linearize_bb_wiod, clear
/*
ren s Xs
keep  ex im exname imname Xs yi yj border ldist contig  comlang colony rta 
save EP_approximation, replace
*/

/*******************************************************************************/
/* Log file */
/*******************************************************************************/
sca cfs= -int(100*$cf)+100
local cfs=cfs
log using Linearize_bb_wiod_`exp'_`cfs',replace

/*******************************************************************************/
/* Final set of variables */
/*******************************************************************************/
keep  ex im exname imname  yi yj border ldist contig  comlang colony rta cu fta  lim lex

/*******************************************************************************/
/* Initializing Variables */
/*******************************************************************************/
gen borderc=0
gen double one=1
gen double za0=0
gen double zac=0
gen double xbb=0
gen double xcbb=0
gen double yp_bb=0
gen double yp_bvols=0
gen double yp_bb_ww=0

gen double ypcf_avw=0
gen double ypcf_avwge=0
gen double ypcf_bb=0
gen double ypcf_bvols=0
gen double ypcf_ww=0
gen double ypcf_wwge=0
gen double psls=0

gen double yp_avw_w=0
gen double yp_bb_w=0
gen double yp_bvols_w=0
gen double ypcf_w=0
gen double yp_ww_w=0
gen double yp_w=0
gen double sp_bbt=0

gen double ypcf_bb_w=0
gen double ypcf_bvols_w=0
gen double ypcf_ww_w=0
gen double ypcf_avw_w=0
gen double ypcf_avwge_w=0
gen double ypcf_wwge_w=0

gen double dypcf_avw=0
gen double dypcf_avwge=0
gen double dypcf_bvols=0
gen double dypcf_w=0
gen double dypcf_wwge=0

gen double dypcf_avw_w=0
gen double dypcf_avwge_w=0
gen double dypcf_bb_w=0
gen double dypcf_bvols_w=0
gen double dypcf_ww_w=0
gen double dypcf_wwge_w=0

gen double dypcf_dir_w=0
gen double dypcf_dir_avw=0

gen double ypbb_avwge=0
gen double ypt_avw=0

/*******************************************************************************
/* Descriptives */
********************************************************************************/
sum  $re
sum  $re  if ex~=im
sum  $re  if ex==im

/*******************************************************************************/
/* 0. true model is PPML*/
/*******************************************************************************/
gen thx=yi
gen thm=yj
gen thxm=yi*yj

table ex , c(sum   thxm  m thx ) row
table im , c(sum   thxm  m thm ) row

/*
*** simulates parameters ***
1	-2.145215	-2.144369	-2.607016	-2.401684	-2.144871	-2.154290
2	-0.833723	-0.834228	-0.905588	-0.911525	-0.833247	-0.832330
3	-0.287841	-0.285738	-0.133215	-0.174538	-0.287082	-0.284418
4	-0.326669	-0.327888	-0.198584	-0.241149	-0.327542	-0.324533
5	 0.589344	0.591799	0.708167	0.704003	0.589925	0.591478
*/

# delimit ; 
mat B=( -2.145215,	-2.144369,	-2.607016,	-2.401684,	-2.144871,	-2.154290 \
		-0.833723,	-0.834228,	-0.905588,	-0.911525,	-0.833247,	-0.832330 \
		-0.287841,	-0.285738,	-0.133215,	-0.174538,	-0.287082,	-0.284418 \
		-0.326669,	-0.327888,	-0.198584,	-0.241149,	-0.327542,	-0.324533 \
		 0.589344, 	 0.591799,	 0.708167,	 0.704003,	 0.589925,	 0.591478 
);
# delimit cr 
mat li B


mat b_t  =B[1..5,1]
mat b_avw=B[1..5,2]
mat b_bb =B[1..5,3]     /*true  GDP share -> changed in mc file*/
mat b_bvols =B[1..5,4]  /*observed  GDP share -> changed in mc file*/
mat b_w  =B[1..5,5]
mat b_ww =B[1..5,6]

mat li b_t
mat li b_bb
mat li b_bvols

gen zat=0
gen za_t=0
gen za_avw=0
gen za_bvols=0
gen za_bb=0
gen za_w=0
gen za_ww=0

gen zacf_t=0
gen zacf_avw=0
gen zacf_bvols=0
gen zacf_bb=0
gen zacf_w=0
gen zacf_ww=0

local i=1
qui foreach z of global re {
display "`z'"
replace za_t=za_t+B[`i',1]*`z'
replace za_avw=za_avw+B[`i',2]*`z'
replace za_bb= za_bb +B[`i',3]*`z'
replace za_bvols=za_bvols+B[`i',4]*`z'
replace za_w=za_w    +B[`i',5]*`z'
replace za_ww=za_ww  +B[`i',6]*`z'
local i=`i'+1
}

/*******************************************************************************/
/* 1. Counterfactuals */
/*******************************************************************************/
/*******************************************************************************/
/* 1.1 uniform ad-valorem trade cost change for cross-border trade  */
/*******************************************************************************/
di as red "experimient: "  "`exp'"
if "`exp'"=="tc" {
replace zacf_t= za_t 
replace zacf_avw=za_avw 
replace zacf_bb=za_bb 
replace zacf_bvols=za_bvols 
replace zacf_w=za_w 
replace zacf_ww=za_ww 

replace zacf_t= za_t-log($cf) if ex~=im 
replace zacf_avw=za_avw-log($cf) if ex~=im  
replace zacf_bb=za_bb-log($cf) if ex~=im  
replace zacf_bvols=za_bvols-log($cf) if ex~=im  
replace zacf_w=za_w-log($cf) if ex~=im  
replace zacf_ww=za_ww-log($cf) if ex~=im  


sum za*_t
sum za*_t if ex==im
sum za_t if ex~=im
sca base=r(mean)
sum zacf_t if ex~=im
sca cf=r(mean)
di "change=" cf-base

sum za*_avw
sum za*_avw if ex==im
sum za*_avw if ex~=im

sum za*_bvols
sum za*_bvols if ex==im
sum za*_bvols  if ex~=im

sum za*_bb
sum za*_bb if ex==im
sum za*_bb if ex~=im

sum za*_w 
sum za*_w if ex==im
sum za*_w if ex~=im

sum za*_ww 
sum za*_ww if ex==im
sum za*_ww if ex~=im

/*tariff equivalent Yotov, Piermartini, Monteiro, Larch Guide*/
di as red "cf-reduction:" cf-base ", tarrif equivalent at s=5:" 100* (exp((cf-base)/(-$s))-1)
}

/*******************************************************************************/

/*******************************************************************************/
/* 1.2 change in lndist  */
/*******************************************************************************/
if "`exp'"=="ldist" {
replace zacf_t= za_t 
replace zacf_avw=za_avw 
replace zacf_bb=za_bb 
replace zacf_bvols=za_bvols 
replace zacf_w=za_w 
replace zacf_ww=za_ww 

replace zacf_t=za_t+         B[2,1]*log($cf) if ex~=im 
replace zacf_avw=za_avw+     B[2,2]*log($cf) if ex~=im 
replace zacf_bb= za_bb+      B[2,3]*log($cf) if ex~=im 
replace zacf_bvols=za_bvols+ B[2,4]*log($cf) if ex~=im 
replace zacf_w=za_w+         B[2,5]*log($cf) if ex~=im 
replace zacf_ww=za_ww+       B[2,6]*log($cf) if ex~=im 
local i=`i'+1

sum za*_t
sum za*_t if ex==im
sum za_t if ex~=im
sca base=r(mean)
sum zacf_t if ex~=im
sca cf=r(mean)
di "change=" cf-base

sum za*_avw
sum za*_avw if ex==im
sum za*_avw if ex~=im

sum za*_bvols
sum za*_bvols if ex==im
sum za*_bvols  if ex~=im

sum za*_bb
sum za*_bb if ex==im
sum za*_bb if ex~=im

sum za*_w 
sum za*_w if ex==im
sum za*_w if ex~=im

sum za*_ww 
sum za*_ww if ex==im
sum za*_ww if ex~=im
/*tariff equivalent Yotov, Piermartini, Monteiro, Larch Guide*/
di as red "cf-reduction:" cf-base ", tarrif equivalent at s=5:" 100* (exp((cf-base)/(-$s))-1)

}

/*******************************************************************************/

/*******************************************************************************/
qui {
*** starting values true models ***
glm thxm ibn.im ib$b.ex , nocons family(poisson) robust offset(za_t)
predict sp
qui sum sp,d
replace sp=sp/r(sum)

drop thx thm thxm yi yj
egen thx=sum(sp),by(ex)
egen thm=sum(sp),by(im)
gen  thxm=thx*thm
gen yi=thx
gen yj=thm
gen ls=log(sp)-log(yi)-log(yj)
gen yt=ls
gen y=ls
gen s=exp(ls+log(yi)+log(yj))

table ex , c(sum   thxm  m thx sum sp sum s) row
table im , c(sum   thxm  m thm sum sp sum s) row
}
/*******************************************************************************/

/*******************************************************************************/
/* Mata invariant matrices and procedures */
/*******************************************************************************/
sort ex im 
qui do mata_mat_wiod.do
do mata_prod_wiod_final.do
/*******************************************************************************/

/*******************************************************************************
/* Check solver for conditional GE at true parameters */
********************************************************************************/
mata 
p=pass()
p.za=za_t
p.thphi=thphi
p.D=D
p.yi=yi
p.yj=yj
end

*** Starting values from Poisson when solving for mui and mj
qui poisson sp $re  ibn.im ib$b.ex , nocons
mat b_start=e(b)'
mata
b_start=st_matrix( "b_start")
start=b_start[$K+1..rows(b_start)-1,1]

S = solvenl_init()
solvenl_init_argument(S,1, p)
solvenl_init_evaluator(S, &avw())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.075)
solvenl_init_startingvals(S, start)
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-20)
solvenl_init_conv_nearzero(S, 1e-20)
solvenl_init_iter_log(S, "on")
v = solvenl_solve(S)
ypt_avw=za_t+D*v
check1= norm(D'*exp(ypt_avw)-thphi)
check1
st_store(. , "ypt_avw",(ypt_avw))
end
gen spt_avw=exp(ypt_avw)
egen yit=sum(spt_avw),by(ex)
egen yjt=sum(spt_avw),by(im)

table ex if ex==im, c(sum yit sum yi) row
table im if ex==im, c(sum yjt sum yj) row
/*******************************************************************************/

/*******************************************************************************/
/* equivalent BB-endowments I*/ 
/*******************************************************************************/
mata mu=v[43..83,1]
mata m=v[1..42,1]
mata thphix= thphi[43..83,1]
/*mata (mu, thphix)*/
/*mata sum(thphix)*/
mata thphix_base=1-sum(thphix)
/*mata thphix*/
mata mu=(mu\0)
mata thphix=(thphix\thphix_base)
mata (mu, thphix)

mata sigma = 5
mata p_avw=exp(mu/(1-sigma))
mata a=thphix:/p_avw
**mata (p_avw,a,p_avw:*a-thphix, thxx)
/******************************************************************************/

/*******************************************************************************/
/* 2. BB full ge at approximazion point t=1*/
/*******************************************************************************/
mata 
zabb=J(rows(Z),1,0)
start=log(thm\thx)
mu0=start[$b+1..rows(start),1]
mu0=(mu0\0)

mata 
pge=pass_ge()
pge.za=zabb
pge.thphi=thphi
pge.D=D
pge.Dm=Dm
pge.Dxx=Dxx
pge.yi=yi
pge.yj=yj
pge.thxx=thxx  /*plug in endowmewnts */
pge.mu0=mu0
pge.s=$s
pge.thm=thm

/*solve ge counterfactual*/
S = solvenl_init()
solvenl_init_argument(S,1, pge)
solvenl_init_evaluator(S, &avwge())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.05)
solvenl_init_startingvals(S, start )
solvenl_init_conv_maxiter(S, 1000)
solvenl_init_conv_iterchng(S,  1e-25)
solvenl_init_conv_nearzero(S, 1e-25)
vbbge = solvenl_solve(S)
ypbb_avwge=zabb+D*vbbge
sp_bbt=exp(D*vbbge)
st_store(. , "ypbb_avwge",(ypbb_avwge)) 
st_store(. , "sp_bbt",(sp_bbt)) 
end

gen double spbb_avwge=exp(ypbb_avwge)
egen double yibb=sum(spbb_avwge),by(ex)
egen double yjbb=sum(spbb_avwge),by(im)

**table ex if ex==im, c(sum yibb sum yi) row
**table im if ex==im, c(sum yjbb sum yj) row
/*******************************************************************************/

/*******************************************************************************/
/* equivalent BB-endowments II*/ 
/*******************************************************************************/
mata mubb=vbbge[43..83,1]
mata mbb=vbbge[1..42,1]

mata thphixbb= Dxx'*sp_bbt /*thphixbb determined endogenously*/
mata mubb=(mubb\0)

mata yitr=Dx'yi/42
mata yitr_base=1-sum(yitr)
mata yitr=(yitr\yitr_base)

mata p_bba=exp(mubb/(1-sigma))

/*
mata e=J(42,1,1)
mata  (thphix, Dxx'(exp(mu)#exp(m)), exp(mu)*1, Dxx'(e#exp(m)) ,  exp(mu):*Dxx'(e#exp(m)) -  Dxx'(exp(mu)#exp(m)))

mata  (thphixbb, exp(mubb)*.2105071404, Dxx'(e#exp(mbb)) ,  exp(mubb):*Dxx'(e#exp(mbb)) -  Dxx'(exp(mubb)#exp(mbb)))

mata ((Dxx'*yi/42):/ thphixbb)
*/

/*******************************************************************************/
/* equivalent BB-endowments III*/ 
/*******************************************************************************/
mata sigma = 5
mata p_bba=exp(mubb/(1-sigma))
mata a=thphixbb:/p_bba
mata a=a
mata (p_bba,a,p_bba:*a-thphixbb) /* bba is correct */

mata astar=(p_bba:/p_avw):*a
mata astar1=( exp(mubb/(1-sigma)):/exp(mu/(1-sigma)) ):*a
mata (astar,a, astar1:/a)
mata (p_bba:*a,  thphixbb, p_avw:*astar,thphixbb)
mata ((p_bba:/p_avw),p_bba:*a,  thphixbb, p_avw:*astar1,thphixbb, astar-a)

preserve
clear
set obs 42
gen a=0
gen astar=0
mata st_store(. , "a",(a)) 
mata st_store(. , "astar",(astar)) 

mata (sum(a), sum(astar), min(astar-a), mean(astar-a), max(astar-a))
gen da=astar-a
sum

**hist da, bin(20)

label var a "A" 
label var  da"A*-A"
/*
scatter da a, ///
		ytitle("A*-A",size(small) margin(medium)) ///
		xtitle("True endowment, A",size(small) margin(medium)) ///
		msize(small small small small ) ///
        msymbol(o o o o) mlwidth( vthin vthin vthin vthin  ) ///
        mfcolor(gs8 gs16) mlcolor(gs1 gs5 gs5 gs5)   yline(0,  lcolor(gs1) )  ///		  
		legend( rows(1)  region(lstyle(none)) size(small) ) scheme( s1mono)  ///
		xsize(4) ysize(4)
*/
restore
/*******************************************************************************/

/*******************************************************************************/
/* 3. AVW in levels*/
/*******************************************************************************/
*** check AV-estimates
qui glm thxm ibn.im ib$b.ex  , nocons offset(za_avw) family(poisson) irls vce(robust) 
predict thxmp
gen yp_avw=log(thxmp)

table ex , c(sum thxmp sum thxm m thx) row
table im , c(sum thxmp sum thxm m thm) row

** b_avw from Monte Carlos
mat outb=  (b_w,  b_avw )
mat li outb
mata yp_avw_test=st_data(., "yp_avw")
/*******************************************************************************/

/*******************************************************************************/
/* 4. Solve AVW using b_w  */
/*******************************************************************************/
*** za = Index from structural parameters za_w
*** Solve AVW-trade resistance equations at yi and yj					
glm thxm ibn.im ib$b.ex  , nocons offset(za_w) family(poisson) irls vce(robust) 
predict sp_w
gen y_awv_w=log(sp_w)
mat mu1=e(b)'
mata mu1=st_matrix( "mu1")

table ex , c(sum sp_w sum thxmp sum thxm m thx) row
table im , c(sum sp_w sum thxmp sum thxm m thm) row

mata yp_avw_w_test=st_data(., "yp_avw_w")
mata sp_w=st_data(., "sp_w") /*for mata predictions*/
/*******************************************************************************/

/*******************************************************************************
/* 5. Base and counterfactuals with linear approximation */
********************************************************************************/

/*******************************************************************************/
/* 5.1 Intermediate step: Dummy model */
/*******************************************************************************/
qui reg ls $re  ibn.im ib$b.ex , nocons r
predict pls_w
predict rr, res
gen  y_fe=pls_w+log(yi)+log(yj)

*** within-parameters long 
mat b_w1=e(b)'
mata b_w1=st_matrix( "b_w1")
mata b_w1=b_w1[1.. rows(b_w1)-1,1]

/*******************************************************************************/
/* 5.2 Mata for predictions */
/*******************************************************************************/
mata 
b_avw=st_matrix("b_avw")
b_wz=b_w[1..$K,1]
b_we=st_matrix("b_we")
b_bb=st_matrix("b_bb")
b_bvols=st_matrix("b_bvols")
b_ww=st_matrix( "b_ww")

Gbb=diag(sp_bbt)
Gbvols=diag(yi:*yj)
Gw=diag(sp_w)
Qbb=I($b^2)- D*invsym(D'*Gbb*D)*D'*Gbb
Qbvols=I($b^2)- D*invsym(D'*Gbvols*D)*D'*Gbvols 
Qw=I($b^2)- D*invsym(D'*D)*D' 
Qww=I($b^2)- D*invsym(D'*Gw*D)*D'*Gw

mu=b_w1[$b+$K+1..rows(b_w1),1]
mu=(mu\0)

/*
mu1=mu1[$b+1..rows(mu1),1]
(mu, mu1)
norm(mu-mu1)

function predict  (real matrix Qbb, matrix Qbvols, real matrix Qw,
	real matrix Z, real matrix Z0, 
	real matrix sp, real matrix sp_w, real matrix sp_bbt, real matrix D, 
	real matrix b_w,
	real matrix za_avw, real matrix za_bvols,  real matrix za_bb, 
	real matrix za_w, real matrix za_ww, 
	real matrix zacf_avw, real matrix zacf_bvols,  real matrix zacf_bb, 
	real matrix zacf_w, real matrix zacf_ww,
	real matrix thm , real matrix thx, real matrix thxx, 
	real matrix y, real matrix yi ,  real matrix yj,  real vector mu )
*/
	   
dypcf= predict(Qbb, Qbvols, Qw,  Z, Z0,  ///
			   sp, sp_w, sp_bbt, D, b_w, /// 
			   za_avw, za_bvols, za_bb, za_w, za_ww, ///
			   zacf_avw, zacf_bvols, zacf_bb, zacf_w, zacf_ww, ///			   
			   thm, thx, thxx, y, yi, yj,mu )
end

/*******************************************************************************/
/* 6. Run nonlinear AvW with b_avw */
/*******************************************************************************/
mata 
za=za_avw
zacf=zacf_avw

p=pass()
p.za=za
p.thphi=thphi
p.D=D
p.yi=yi
p.yj=yj
end

/*******************************************************************************/
/* 6a. Base wit b_avw*/
/*******************************************************************************/
*** Starting values from Poisson when solving for mui and mj
qui poisson s $re  ibn.im ib$b.ex , nocons
mat b_pw=e(b)'
**mat b_pw=b_w
mat b_ws=b_w[1..5,.]
mat b_pw[1,1]=b_ws
**mat li b_pw

mata
b_pw=st_matrix( "b_pw")
start=b_pw[$K+1..rows(b_pw)-1,1]

/*solve base*/
S = solvenl_init()
solvenl_init_argument(S,1, p)
solvenl_init_evaluator(S, &avw())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.075)
solvenl_init_startingvals(S, start)
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-10)
solvenl_init_conv_nearzero(S, 1e-10)
solvenl_init_iter_log(S, "on")
v = solvenl_solve(S)
yp_avw=za+D*v
check1= norm(D'*exp(yp_avw)-thphi)
st_store(. , "yp_avw",(yp_avw))
 
/* base for ge mill prices */
mu0=v[$b+1..rows(v),1]
mu0=(mu0\0)

/*******************************************************************************/
/* 6b. Counterfactual conditional ge with b_avw */
/*******************************************************************************/
pcf=pass()
pcf.za=zacf
pcf.thphi=thphi
pcf.D=D
pcf.yi=yi
pcf.yj=yj

/*solve counterfactual*/
S = solvenl_init()
solvenl_init_argument(S,1, pcf)
solvenl_init_evaluator(S, &avw())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.075)
solvenl_init_startingvals(S, start)
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-10)
solvenl_init_conv_nearzero(S, 1e-10)
solvenl_init_startingvals(S, start)
v = solvenl_solve(S)
vcf=v
ypcf_avw=zacf+D*vcf
check2=norm(D'*exp(ypcf_avw)-thphi)
st_store(. , "ypcf_avw",(ypcf_avw)) 
dypcf_avw=-(yp_avw-ypcf_avw)
dypcf_dir_avw=-(za-zacf)
mu=v[$b+1..rows(v),1]
mu=(mu\0)
(check1,check2)
end

/*******************************************************************************/
/* 6b. Counterfactual full ge*/
/*******************************************************************************/
mata 
pge=pass_ge()
pge.za=zacf
pge.thphi=thphi
pge.D=D
pge.Dm=Dm
pge.Dxx=Dxx
pge.yi=yi
pge.yj=yj
pge.thxx=thxx
pge.mu0=mu0
pge.s=$s
pge.thm=thm

/*solve ge counterfactual*/
S = solvenl_init()
solvenl_init_argument(S,1, pge)
solvenl_init_evaluator(S, &avwge())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.05)
solvenl_init_startingvals(S, v )
solvenl_init_conv_maxiter(S, 100)
solvenl_init_conv_iterchng(S,  1e-15)
solvenl_init_conv_nearzero(S, 1e-15)
vge = solvenl_solve(S)
ypcf_avwge=zacf+D*vge
st_store(. , "ypcf_avwge",(ypcf_avwge)) 
dypcf_avwge=-(yp_avw-ypcf_avwge)
end
/*******************************************************************************/

/*******************************************************************************
/* 7. Run AvW with b_w */
********************************************************************************/
mata 
za=za_w
zacf=zacf_w

p=pass()
p.za=za
p.thphi=thphi
p.D=D
p.yi=yi
p.yj=yj
end

/*******************************************************************************/
/* 7a. Base */
/*******************************************************************************/
*** Starting values from Poisson when solving for mui and mj
qui poisson s $re  ibn.im ib$b.ex , nocons
mat b_pw=e(b)'
**mat b_pw=b_w
mat b_ws=b_w[1..5,.]
mat b_pw[1,1]=b_ws
**mat li b_pw

mata
b_pw=st_matrix( "b_pw")
start=b_pw[$K+1..rows(b_pw)-1,1]

/*solve base*/
S = solvenl_init()
solvenl_init_argument(S,1, p)
solvenl_init_evaluator(S, &avw())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.075)
solvenl_init_startingvals(S, start)
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-10)
solvenl_init_conv_nearzero(S, 1e-10)
solvenl_init_iter_log(S, "on")
v = solvenl_solve(S)
yp_avw_w=za+D*v
check1= norm(D'*exp(yp_avw_w)-thphi)
check1
st_store(. , "yp_avw_w",(yp_avw_w))
 
/* base for ge mill prices */
mu0=v[$b+1..rows(v),1]
mu0=(mu0\0)
end

/*******************************************************************************/
/* 7b. Counterfactual conditional ge with b_wz */
/*******************************************************************************/
mata
p=pass()
p.za=zacf
p.thphi=thphi
p.D=D
p.yi=yi
p.yj=yj

/*solve counterfactual*/
solvenl_init_conv_maxiter(S, 150)
solvenl_init_conv_iterchng(S,  1e-10)
solvenl_init_conv_nearzero(S, 1e-10)
solvenl_init_startingvals(S, start)
v = solvenl_solve(S)
ypcf_avw_w=zacf+D*v
check2=norm(D'*exp(ypcf_avw_w)-thphi)
st_store(. , "ypcf_avw_w",(ypcf_avw_w)) 
dypcf_avw_w=-(yp_avw_w)+(ypcf_avw_w)
dypcf_dir_w=za-zacf
mu=v[$b+1..rows(v),1]
mu=(mu\0)
(check1,check2)
end

/*******************************************************************************/
/* 7c. Counterfactual full ge with b_wz*/
/*******************************************************************************/
mata 
pge=pass_ge()
pge.za=zacf
pge.thphi=thphi
pge.D=D
pge.Dm=Dm
pge.Dxx=Dxx
pge.yi=yi
pge.yj=yj
pge.thxx=thxx
pge.mu0=mu0
pge.s=$s
pge.thm=thm

/*solve ge counterfactual*/
S = solvenl_init()
solvenl_init_argument(S,1, pge)
solvenl_init_evaluator(S, &avwge())
solvenl_init_type(S, "zero")
solvenl_init_technique(S, "broydenpowell")
solvenl_init_numeq(S, $eqs)
solvenl_init_damping(S, 0.05)
solvenl_init_startingvals(S, v )
solvenl_init_conv_maxiter(S, 100)
solvenl_init_conv_iterchng(S,  1e-15)
solvenl_init_conv_nearzero(S, 1e-15)
vge = solvenl_solve(S)
ypcf_avwge_w=zacf+D*vge
st_store(. , "ypcf_avwge_w",(ypcf_avwge_w)) 
dypcf_avwge_w=-(yp_avw_w)+(ypcf_avwge_w)
end
/*******************************************************************************/

/*******************************************************************************/
/* 8. Collect mata results */
/*******************************************************************************/
/*mata dypcf=(dypcf_bb, dypcf_w, dypcf_wwge, ypcf_bb_w, dypcf_ww_w, dypcf_wwge_w)*/

mata
st_store(. , "dypcf_avw",dypcf_avw) 
st_store(. , "dypcf_avwge",dypcf_avwge) 
st_store(. , "dypcf_avw_w",dypcf_avw_w) 
st_store(. , "dypcf_avwge_w",dypcf_avwge_w)

st_store(. , "dypcf_bvols",dypcf[.,1]) 
st_store(. , "dypcf_w",dypcf[.,2]) 
st_store(. , "dypcf_wwge",dypcf[.,3]) 

st_store(. , "dypcf_bvols_w",dypcf[.,4]) 
st_store(. , "dypcf_ww_w",dypcf[.,5]) 
st_store(. , "dypcf_wwge_w",dypcf[.,6]) 
end 


drop dypcf_ww_w /* not properly centered */
drop dypcf_wwge_w /* not properly centered */
drop  dypcf_bb_w /*not calculated */

/*******************************************************************************/
/* 9. Collect est. parameters */
/*******************************************************************************/
mat outb0=  (b_t, b_avw, b_bb, b_bvols, b_w, b_ww)
mat rownames outb0 = $re
mat colnames outb0 = b_true b_avw b_bb b_bvols  b_w b_ww 
estout matrix(outb0, fmt(%10.3f))

mat scale=diag(b_t)
mat outpb0=  (b_avw, b_bb, b_bvols,b_w, b_ww)
mat outpb0= 100*(inv(scale)* outpb0-J(5,5,1))
mat rownames outpb0 = $re
mat colnames outpb0 = b_avw b_bb b_bvols b_w b_ww 
estout matrix(outpb0, fmt(%10.2f))

/*******************************************************************************/
/* 10. Aggregation to country groups */
/*******************************************************************************/
if "`exp'"~="rta" {

qui {
xtile sizext=yi , nq(2)
tab sizext
egen sizex=mean(sizext),by(ex)
tab sizex 
table sizex , c(m yi n yj) 

xtile sizemt=yj , nq(2)
tab sizemt 
egen sizem=mean(sizemt),by(im)
tab sizem 
table sizem , c(m yi n yj) 

list ex im sizex sizem if ex==im & sizex ~= sizem

gen co = 1 if  sizem==1 & ex==im 
replace co = 2  if sizem==2 & ex==im 
replace co = 3 if  sizex==1 & sizem==1 & ex~=im
replace co = 4 if  sizex==2 & sizem==2 & ex~=im
replace co = 5 if  sizex==1 & sizem==2 & ex~=im
replace co = 6 if  sizex==2 & sizem==1 & ex~=im

label define lco 1 "dom-small", add
label define lco 2 "dom-large", add
label define lco 3 "small-small", add
label define lco 4 "large-large", add
label define lco 5 "small-large", add
label define lco 6 "large-small", add

label values co lco 
}
}

if "`exp'"=="rta" {

qui {
gen co = 1 if  ex==im 
replace co = 2 if  rta==1 & ex~=im
replace co = 3 if  co==. & ex~=im

label define lco 1 "dom", add
label define lco 2 "rta", add
label define lco 3 "no rta", add
label values co lco 
}

tab co rta 
}

/*******************************************************************************/
/* 11. Bias and MSE relativ to yn rest skipped */
/*******************************************************************************/
gen yn=yt+log(yi)+log(yj)
local list "avw bb bvols w "
foreach x of local list {
display "`x'"
gen bias_`x'=(yp_`x'-yn)
gen mse_`x'=(yp_`x'-yn)^2
}

local list "avw bb bvols  ww "
foreach x of local list {
display "`x'"
gen bias_`x'_w=(yp_`x'_w-yn)
gen mse_`x'_w=(yp_`x'_w-yn)^2
}

/*******************************************************************************/
/* 12. Results for log trade flow predictions */ 
/*******************************************************************************/

/*******************************************************************************/
/* Paramter estimates */
/*******************************************************************************/
estout matrix(outb, fmt(%10.3f))

/*******************************************************************************/
di as gre "base"
/*******************************************************************************/
gen  weight=exp(yp_avw)
gen yna=abs(yn)

label var yn "Obs"
label var yp_avw  "avw"
label var yp_bvols  "bvols"
label var yp_bb "bb"
label var yp_w  "w"

label var yp_avw_w    "avw_w"
label var yp_bvols_w  "bvols_w"
label var yp_bb_w      "bb_w"
label var yp_ww_w      "ww_t"
**label var yp_ww     "w_w"

/*******************************************************************************/
/* Final Tables for base prediction  */ 
/*******************************************************************************/
/* Abbreviations
yn... true bilateral trade flow/World expenditures
avw...Anderson and van Wincoop exact solution to the non-linear system of multilateral resistances; non-linear iterative estimator
bb... Bayer Bergstrand BVOLS using true GDP shares at t=1
bvols... Bayer Bergstrand BVOLS using observed GDP shares at t=1
ww... weighted within estimator: Projection at base line BVOLS, but left hand side transformed as well
_w use the within estimator (includes export and import dummies) for all specifications
bias is prediction - observed of(logs)
_ge means general equilibrium adjustment of sales and expenditure shares 
*/



/*
(i)  bvols ...Bvols observed GDP-shares 
(ii) bb .. BB true GDP shares
(iii) w...approximation at true trade costs and true GDP-share using  within alpha  yp_w
(iv) bb_w , bb predictions within alpha-hat true theta at t=1: yp_bb_w
*/

********************************************************************************/
*** Parameter estimates ***
********************************************************************************/
*** Bias of estimated parameters ***
estout matrix(outb0, fmt(%10.3f))
*** Deviation of estimated parameters from true ones in % ***
estout matrix(outpb0, fmt(%10.2f))

********************************************************************************/
*** baseline predictions and bias decomposition ***
********************************************************************************/
*** Predictions  ****
version 16
table co , c(m yp_avw m yp_bb  m yp_bvols  m yp_w m yp_ww_w   ) row  center format(%6.3f) cellwidth(10) 
table co , c(m  bias_avw m  bias_bb   m  bias_bvols  m  bias_w   m  bias_ww_w   ) row  center format(%8.3f) cellwidth(10)

********************************************************************************/
di as gre "baseline predictions using the b_w in all models"
********************************************************************************/
version 16
table co [aweight=weight], c(m yn m yp_avw_w m yp_bvols_w  m yp_ww_w   ) row  center format(%8.3f) cellwidth(16)
table co [aweight=weight], c(m  bias_avw_w m  bias_bvols_w  m  bias_ww_w   ) row  center format(%8.3f) cellwidth(16)

********************************************************************************/
di as gre "counterfactual experiment with fixed sales and expenditure shares: "  "`exp'"
********************************************************************************/
table co [aweight=weight], c(m ypcf_avw m ypcf_bvols  m ypcf_ww_w) row  center format(%8.3f) cellwidth(16)

********************************************************************************/
di as gre "counterfactual experiment under  general eqilibrium: "  "`exp'"
di as gre "under general eqilibrium dypcf_wwge still unclear"
********************************************************************************/
table co [aweight=weight], c(m ypcf_avwge_w m ypcf_wwge_w ) row  center format(%8.3f) cellwidth(16)

********************************************************************************/
di as gre "counterfactual changes experiment with fixed sales and expenditure shares: "  "`exp'" , "observed - counterfactual"
********************************************************************************/
table co [aweight=weight], c( m dypcf_avw m dypcf_w m dypcf_bvols  ) row  center format(%8.3f) cellwidth(16)
table co [aweight=weight], c( sd dypcf_avw sd dypcf_w sd dypcf_bvols  ) row  center format(%8.3f) cellwidth(16)

********************************************************************************/
di as gre "counterfactual changes experiment under  general eqilibrium: "  "`exp'" , "observed - counterfactual"
di as gre "under general eqilibrium dypcf_wwge still unclear"
********************************************************************************/
table co [aweight=weight], c(m dypcf_avw m dypcf_avwge m dypcf_wwge ) row format(%8.3f) cellwidth(17)
table co , c(m dypcf_avw m dypcf_avwge m dypcf_wwge ) row format(%8.3f) cellwidth(17)

version 16
table co, c(m sp_w m dypcf_w m dypcf_wwge ) row  center format(%12.8f) cellwidth(20)


sca cfs= -int(100*$cf)+100
local cfs=cfs

if "`exp'" == "ldist" {
sca cfs= -int(100*$cf)+100
local cfs=cfs
}	


save linearize_BB_wiod_`exp'_`cfs'_estimates, replace

exit
