clear all
macro drop all
capture log close 
set more off
set matsize 10000
set linesize 120
set pagesize 80
set seed 1111
set maxvar 10000
program drop _all
clear mata
version 17

**cd "C:\Users\Administrator\Dropbox\Linearize_BB\JORS_Published\stata_final"
cd "C:\Users\mpfaf\Dropbox\Linearize_BB\JORS_Published\stata_final"
global b=42 /* 42 wiod countries */

/*******************************************************************************/
/* Experiment */
/*******************************************************************************/ 
/*ldist cf*/
local exp="tc"
sca exp="tc"

**local exp="ldist"
**sca exp="ldist"

local cfs=5
local cfss=5
sca cfs=5

local cfss=25
local cfs=25
sca cfs=25
*/

**local cfss=1
**local cfs=1
*sca cfs=1


** qui{
/*******************************************************************************/ 
/* Data file */
/*******************************************************************************/
use linearize_BB_wiod_`exp'_`cfss'_estimates, clear

/*******************************************************************************/
/* Log file */
/*******************************************************************************/
log using Linearize_bb_wiod_graphs_`exp'_`cfs',replace

di as red "Experiment " exp " " cfs
  
global re=  "c.ldist i.border  i.contig i.comlang i.rta "

label var yn "true prediction" 
label var  bias_avw "AvW"
label var  bias_bb "BB, true shares"
label var  bias_bvols  "BvOLS, obs. shares"
label var  bias_ww_w  "Weighted within, true shares"
label var  bias_w  "Within, no weights"

label def b 0 "domestic trade"
label def b 1 "international trade", add
label values border b

**correction I
constraint 1 za_w=1
qui cnsreg ls za_w  ibn.im ib$b.ex , nocons constraints(1)
predict yp_w1
replace  yp_w1 = yp_w1+log(yi)+log(yj)

**correction II
constraint 1 za_w=1
qui cnsreg ls za_t  ibn.im ib$b.ex , nocons constraints(1)
predict yp_t1
replace  yp_t1 = yp_t1+log(yi)+log(yj)

sum yp_w yp_w1 yp_t1  yn za_w za_t
sum bias_bvols bias_bb  bias_avw    bias_w   bias_ww_w

/*******************************************************************************/
/* Figure 1 */
/*******************************************************************************/
scatter bias_bvols  bias_bb  yn, ///
		yscale(range(-2 (1) 2))  ylabel(-2(1)2) ///
		ytitle("bias log difference",size(small) margin(medium)) ///
		xtitle("true prediction",size(small) margin(medium)) ///
		msize(small small small small ) ///
        msymbol(o o o o) mlwidth( vthin vthin vthin vthin  ) ///
        mfcolor(gs8 gs16) mlcolor(gs1 gs5 gs5 gs5)   yline(0,  lcolor(gs1) )  ///		  
		legend( rows(1)  region(lstyle(none)) size(small) ) scheme( s1mono)  ///
		xsize(4) ysize(4)

graph save g1.gph, replace	


/*******************************************************************************/
** counterfactuals */
/*******************************************************************************/
*** bias
**drop bdypcf*
gen bdypcf_avw=dypcf_avw-dypcf_avwge
gen bdypcf_bvols=dypcf_bvols-dypcf_avwge
gen bdypcf_w=dypcf_w-dypcf_avwge
gen bdypcf_wwge=dypcf_wwge-dypcf_avwge

version 16
table co ,c( m bdypcf_bvols sd  bdypcf_bvols  m bdypcf_w sd bdypcf_w ) format(%6.3f) row
table co [aweight=sp],c( m bdypcf_bvols sd  bdypcf_bvols  m bdypcf_w sd bdypcf_w ) format(%6.3f) row

label var yn "true prediction" 
label var bdypcf_avw  "AvW, constant shares "
label var dypcf_avwge  "True general equilibrium"
label var bdypcf_bvols  "BvOLS"
label var bdypcf_w   "EP1"

*** Figure 2
if exp== "tc" {
sum  bdypcf_w  bdypcf_bvols dypcf_avwge
graph twoway scatter  bdypcf_w  bdypcf_bvols dypcf_avwge , scheme( s1mono) ///
    title("`cfs' percent ad-valorem trade cost reduction", size(medsmall)) ///
	yscale(range(-0.4 (0.05) 0.2))  ylabel(-1(1)1) ///
	msize(small small small small small) ///
	msymbol(o o o o o) mlwidth( vthin vthin vthin vthin vthin  ) ///
	mfcolor(gs8 gs16) mlcolor(gs2 gs5 gs5 gs5 gs5)   yline(0,  lcolor(gs2) ) ///
	legend( rows(1)  region(lstyle(none))   textwidth(35pt)   size(vsmall)   symxsize(10pt)   ) ///
	xsize(5) ysize(4) ///
	xtitle("true counterfactual change in log trade flows in general equilibrium" , size(vsmall) margin(medium)) ///
	ytitle("bias, log difference counterfactual vs. base" , size(vsmall) margin(medium) ) ///
	xlabel(-0.4 (0.1) 0.4, labsize(vsmall)       )   ///
	ylabel(-0.4 (0.05) 0.2, labsize(vsmall) ) 
	 
graph save g6_`exp'_`cfs'.gph, replace	 
}

if exp== "ldist" {
sum  bdypcf_w  bdypcf_bvols dypcf_avwge
graph twoway scatter  bdypcf_w  bdypcf_bvols dypcf_avwge , scheme( s1mono) ///
    title("`cfs' percent reduction in distance", size(medsmall)) ///
	yscale(range(-0.4(0.05) 0.2))  ylabel(-1(1)1) ///
	msize(small small small small small) ///
	msymbol(o o o o o) mlwidth( vthin vthin vthin vthin vthin  ) ///
	mfcolor(gs8 gs16) mlcolor(gs2 gs5 gs5 gs5 gs5)   yline(0,  lcolor(gs2) ) ///
	legend( rows(1)  region(lstyle(none))   textwidth(35pt)   size(vsmall)   symxsize(10pt)   ) ///
	xsize(5) ysize(4) ///
	xtitle("true counterfactual change in log trade flows in general equilibrium" , size(vsmall) margin(medium)) ///
	ytitle("bias, log difference counterfactual vs. base" , size(vsmall) margin(medium) ) ///
	xlabel(-0.3 (0.05) 0.3, labsize(vsmall)       )   ///
	ylabel(-0.4 (0.05) 0.2, labsize(vsmall) ) 
	 
graph save g6_`exp'_`cfs'.gph, replace	 
}


version 16
table co ,c( m dypcf_bvols sd  dypcf_bvols  m dypcf_w sd dypcf_w ) format(%6.3f) row
table co ,c( m dypcf_avwge sd dypcf_avwge m dypcf_wwge sd dypcf_wwge ) format(%6.3f) row

*** Table in the text ***
table co ,c( m dypcf_bvols   m dypcf_w m dypcf_avwge ) format(%6.4f) row
table co ,c( m dypcf_avwge m bdypcf_bvols sd  bdypcf_bvols  m bdypcf_w sd bdypcf_w ) format(%6.4f) row

if `cfs'== 5 exit

graph combine g6_`exp'_5.gph  g6_`exp'_25.gph  , rows(1) ///
      ycommon xcommon scheme( s1mono) xsize(3) ysize(2) 
graph save g6_`exp'.gph, replace



